<?php


/**
 * CNAB.
 *
 * @package Logic
 *
 * @author  thalesvalias
 */
class CNAB{

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_mesCompetencia;
    private $_dtInicio;
    private $_dtFim;
    private $_dtAux;

    private $_cnabMagnetico;
    private $_cnabMagneticoTrab;
    private $_valorTotalPagamento;
    private $_cnabSantander;
    private $_cnabBrasil;
    private $_registroCNAB;

    private $_itemCalculoFolha;
    private $_evento;
    private $_eventoPagamento;
    private $_pessoa;
    private $_objPessoa;
    private $_vinculo;
    private $_nrTipoHierarqOrg;
    private $_movimentacao;
    private $_estrutura;
    private $_estruturacao;
    private $_nrEstruturaResp;
    private $_dtPagamento;
    private $_nrCalculoFolha;

    private $_cdBancoFormulario;
    private $_nrTpMovimentoFormulario;
    private $_enderecoParc;
    private $_municipio;
    private $_bairro;
    private $_contaCorrenteParceiro;



    const TIPO_TRANSFERENCIA_LEGAL = '1';


    /**
     * Construtor.
     *
     * @param integer             $nrOrg          Numero da Organizacao.
     * @param integer             $nrOrgPadrao    Numero da Organizacao Padrao.
     * @param TkDateTime          $mesCompetencia Mes de competencia.
     * @param FpacnabmagnetEntity $cnabMagnetico  Instancia de FpacnabmagnetEntity.
     * @param FpaformcnabEntity   $formulario     Formulario.
     * @param integer             $nrCalculoFolha Numero do calculo de folha
     *
     * @author thalesvalias - 27/07/2012
     */
    public function __construct(
        $nrOrg, $nrOrgPadrao, TkDateTime $mesCompetencia, FpacnabmagnetEntity $cnabMagnetico,
        FpaformcnabEntity $formulario, $nrCalculoFolha
    ){
        $this->_nrOrg = $nrOrg;
        $this->_nrOrgPadrao = $nrOrgPadrao;
        $this->_mesCompetencia = $mesCompetencia;
        $this->_evento = new FpaeventomEntity(null, $this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia);

        $this->_dtInicio = new TkDateTime($this->_mesCompetencia);
        $this->_dtFim = new TkDateTime($this->_mesCompetencia);
        $this->_dtFim->addMonth(1)->subDay(1);
        $this->_dtAux = new TkDateTime();

        $this->_itemCalculoFolha = new FpaitecalcfolhaEntity();
        $this->_objPessoa = new GpepessoaEntity(null, $this->_nrOrg, $this->_mesCompetencia);
        $this->_vinculo = new GpevinculomEntity();
        $this->_movimentacao = new GpemovimentacaoEntity();

        $this->_enderecoParc = new EnderecoparcEntity();
        $this->_municipio = new MunicipioEntity();
        $this->_bairro = new BairroEntity();
        $this->_contaCorrenteParceiro = new ContcorrparcEntity(null, $this->_nrOrgPadrao, $this->_nrOrg);

        $this->_cnabSantander = new CNABSantander($this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia, $cnabMagnetico, $formulario);
        $this->_cnabBrasil = new CNABBrasil($this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia, $cnabMagnetico, $formulario);
        $this->_registroCNAB = new RegistroCNAB();

        $this->_nrCalculoFolha = $nrCalculoFolha;

        $this->_estruturacao = new Estruturacao($this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia);
        $this->_estrutura = new EstruturamEntity(null, $this->_nrOrg, $this->_mesCompetencia);

        $this->_cnabMagnetico = $cnabMagnetico;
        $this->_cnabMagneticoTrab = new FpacnabmagnetEntity();

        $this->_inicializaVariaveis($formulario);
    }


    private function _inicializaVariaveis($formulario){

        $this->_estruturacao->carregaHierarquias($formulario->getNrhierarqorg());
        $this->_estruturacao->carregaTipoHierarquia($formulario->getNrhierarqorg());
        $this->_cdBancoFormulario = $formulario->getCdbanco();
        $this->_nrTpMovimentoFormulario = $formulario->getNrtipomoviment();
        $this->_nrTipoHierarqOrg = $formulario->getNrhierarqorg();
        $this->_nrEstruturaResp = $formulario->getNrestrutrespons();
        $this->_dtPagamento = $formulario->getDtpagamento();
        // Retorna o evento de acordo com o numero de evento setado no formulario.
        $this->_eventoPagamento = $this->_evento->retornaEvento($formulario->getNreventom());
        $this->_valorTotalPagamento = 0;
    }


    /**
     * Cria os registros CNAB de acordo com o banco informado no formulario.
     *
     * @param GpevinculomEntity $vinculo Instancia de GpevinculomEntity.
     *
     * @return void
     *
     * @author thalesvalias - 01/08/2012
     */
    public function preparaCNAB(GpevinculomEntity $vinculo){

        $this->_vinculo = $vinculo;
        $this->_pessoa = $this->_objPessoa->retornaPessoaAtual($this->_vinculo->getNrpessoa());
        $this->_preparaEstrutura();

        // Apenas gera o CNAB se o banco da estrutura do vinculo informado for o banco informado no formulario
        // e se o banco da estrutura do vinculo informado for o banco do vinculo informado.
//        if ($this->_estrutura->banco === $this->_cdBancoFormulario) {
            $this->_preparaBancoPessoa();
            if ($this->_pessoa->banco === $this->_cdBancoFormulario) {
//            if ($this->_pessoa->banco === $this->_estrutura->banco) {
                $this->_preparaPessoa();
                $this->_preparaVinculo();

                // Verifica qual eh o banco da estrutura do vinculo informado.
                switch($this->_estrutura->banco) {
                    case CNABSantander::CODIGO_BANCO_SANTANDER:
                        $this->_cnabSantander->geraCNAB($this->_estrutura, $this->_pessoa, $this->_vinculo);
                        break;
                    case CNABBrasil::CODIGO_BANCO_DO_BRASIL:
                        $this->_cnabBrasil->geraCNAB($this->_estrutura, $this->_pessoa, $this->_vinculo);
                        break;
                }
            }else{
                tk_warning('CNAB nao gerado para Funcionário '.$this->_vinculo->getNmvinculom().' por nao ter conta corrente parametrizado.');
            }

//        } else{
//            tk_warning('Parceiro de negócio definido na estrutura ' . $this->_pessoa->getNrparcnegocio() . ' nao possui conta corrente ou banco diferente do selecionado no formulario.');
//        }
    }


    /**
     * Prepara os dados da estrutura do vinculo informado.
     *
     * @return void
     *
     * @author thalesvalias - 01/08/2012
     */
    private function _preparaEstrutura() {
        // alterado para retorna os dados da estrutura responsavel informada no formulário - leonardocarvalho 26/08/2014
         $estrutura = $this->_estruturacao->retornaEstrutura($this->_nrEstruturaResp);
         if($estrutura){
             $this->_estrutura = $estrutura;
             $this->_estrutura->historico = $estrutura->getHistorico();

        // Movimentacao corrente.
//        $movimentacao = $this->_estruturacao->getMovimentacaoVinculoPorTipo($this->_vinculo->getNrvinculom(), self::TIPO_TRANSFERENCIA_LEGAL);
//
//        if (is_object($movimentacao) === true) {
//            // Estrutura superior do vinculo.
//            $this->_estrutura = $this->_estruturacao->getEstruturaSuperior($movimentacao);
//            $estrutura = $this->_estruturacao->retornaEstrutura($this->_estrutura->getNrestruturam());
//            $this->_estrutura->historico = $estrutura->getHistorico();

        } else {
            $this->_estrutura = null;
            tk_warning('Estrutura '.$this->_estrutura->getNrestruturam().' sem conta parâmetrizada');
        }

        if ($this->_estrutura !== null) {

            // Define os dados de banco da estrutura (hierarquia superior).
//            $contaCorrenteEstrutura = $this->_contaCorrenteParceiro->retornaContaCorrentePorParceiro($this->_estrutura->historico->getNrparcnegocio());

            // alterado para buscar a conta corrente da estrutura pelo parceiro de negocio, organização e banco informado no formulário - leonardocarvalho 26/08/2014
            $contaCorrenteEstrutura = \Teknisa\Entities\Contcorrparc::findOneByNrparcnegocioAndNrorgAndCdbanco($this->_estrutura->historico->getNrparcnegocio(), $this->_nrOrg, $this->_cdBancoFormulario);
            if (is_object($contaCorrenteEstrutura) == true) {
                $this->_estrutura->banco = $contaCorrenteEstrutura->getCdbanco();
                $this->_estrutura->agencia = $contaCorrenteEstrutura->getCdagencia();
                $this->_estrutura->contaCorrente = $contaCorrenteEstrutura->getCdcontcorr();
            } else {
                $this->_estrutura->banco = null;
                $this->_estrutura->agencia = null;
                $this->_estrutura->contaCorrente = null;
                throw new Exception('Estrutura '.$this->_estrutura->getNrestruturam().' não possui conta corrente para o banco informado.');
            }

        }

//  //@todo - passando manualmente a conta da nutri, remover depois.
//  $this->_estrutura->banco = '001';
//  $this->_estrutura->agencia = '25828';
//  $this->_estrutura->contaCorrente = '76015';//103284


            // Define o tipo e o numero de inscricao da estrutura (hierarquia superior).
            $cnpjEstrutura = $this->_estrutura->historico->getCdcnpjestrut();
            $cpfEstrutura = $this->_estrutura->historico->getCdcpfestrutura();
            if (empty($cnpjEstrutura) === false) {
                $this->_estrutura->nrTpInscricao = RegistroCNAB::CNPJ;
                $this->_estrutura->nrInscricao = $cnpjEstrutura;
            } elseif (empty($cpfEstrutura) === false) {
                $this->_estrutura->nrTpInscricao = RegistroCNAB::CPF;
                $this->_estrutura->nrInscricao = $cpfEstrutura;
            } else {
                tk_warning('Estrutura sem número de inscrição!');
            }

            // Define os dados referentes ao endereco da estrutura.
            $endereco = $this->_estruturacao->determinaEnderecoEstrutura($this->_estrutura);
            $this->_estrutura->endereco = $endereco;

        // }else{
        //     $this->_estrutura = null;
        //     $this->_estrutura->historico = null;
        // }

    }


    /**
     * Prepara os dados do banco do vinculo(pessoa) informado.
     *
     * @return void
     *
     * @author thalesvalias - 01/08/2012
     */
    private function _preparaBancoPessoa() {
        $contaCorrentePessoa = $this->_contaCorrenteParceiro->retornaContaCorrentePorParceiro($this->_pessoa->getNrparcnegocio());
        if (is_object($contaCorrentePessoa) == true) {
            $this->_pessoa->banco = $contaCorrentePessoa->getCdbanco();
            $this->_pessoa->agencia = $contaCorrentePessoa->getCdagencia();
            $this->_pessoa->contaCorrente = $contaCorrentePessoa->getCdcontcorr();
        } else {
            $this->_pessoa->banco = null;
            $this->_pessoa->agencia = null;
            $this->_pessoa->contaCorrente = null;
        }
    }


    /**
     * Prepara os dados da pessoa referente ao vinculo informado.
     *
     * @return void
     *
     * @author thalesvalias - 01/08/2012
     */
    private function _preparaPessoa(){

        $nrParceiroNegocio = $this->_pessoa->getNrparcnegocio();

        // Tipo de inscricao.
        $this->_pessoa->nrTpInscricao = RegistroCNAB::CPF;

        // Dados referentes ao endereco da pessoa.
        $enderecoParc = $this->_enderecoParc->retornaEnderecoParceiroPorTipo(
            $this->_nrOrg, $nrParceiroNegocio, Estruturacao::ENDERECO_PRINCIPAL
        );

        if (is_object($enderecoParc) === true) {
            // Divide o endereco em endereco, numero e complemento.
            $enderecoParc->separaDsEndereco();

            $municipio = $this->_municipio->retornaMunicipio(
                $enderecoParc->getCdpais(), $enderecoParc->getSgestado(), $enderecoParc->getCdmunicipio()
            );
            if (is_object($municipio) === true) {
                $this->_pessoa->municipio = $municipio->getNmmunicipio();
            } else {
                $this->_pessoa->municipio = null;
            }

            $this->_pessoa->endereco    = $enderecoParc->getDsendereco();
            $this->_pessoa->numero      = $enderecoParc->getNrimovelendereco();
            $this->_pessoa->complemento = ltrim($enderecoParc->getComplemento());
            $this->_pessoa->bairro      = $this->_bairro->defineBairro($enderecoParc);
            $this->_pessoa->estado      = $enderecoParc->getSgestado();
            $this->_pessoa->cep         = $enderecoParc->getNrcependereco();

        } else {
            $this->_pessoa->endereco = null;
            $this->_pessoa->numero = null;
            $this->_pessoa->complemento = null;
            $this->_pessoa->bairro = null;
            $this->_pessoa->municipio = null;
            $this->_pessoa->estado = null;
            $this->_pessoa->cep = null;
        }
    }


    /**
     * Prepara os dados do vinculo informado.
     *
     * @return void
     *
     * @author thalesvalias - 14/08/2012
     */
    private function _preparaVinculo(){

        // Define o valor e a data do pagamento do salario liquido do vinculo.
        $this->_definePagamentoCnab();
    }


    /**
     * Retorna linha do registro modificada com a sequencia correta.
     *
     * @param string  $linha     Linha do registro.
     * @param integer $sequencia Numero da sequencia.
     *
     * @return string (linha modificada do registro)
     *
     * @author thalesvalias - 07/08/2012
     */
    public function modificaSequencia($linha, $sequencia){

        $linha = $this->_registroCNAB->alteraDado(
            $linha, (string)$sequencia, (9 -1), 5, RegistroCNAB::CARACTER_ZERO, RegistroCNAB::PREENCHE_ESQUERDA
        );

        return $linha;
    }


    /**
     * Retorna linha do registro modificada(atualizada) com as consolidacoes de acordo com o tipo de registro.
     *
     * @param integer $tipoRegistro Tipo do registro da linha.
     * @param string  $linha        Linha do registro.
     *
     * @return string (linha modificada do registro)
     *
     * @author thalesvalias - 07/08/2012
     */
    public function modificaLinhasConsolidacao($tipoRegistro, $linha){

        switch($this->_cdBancoFormulario) {
            case CNABSantander::CODIGO_BANCO_SANTANDER:
                $linha = $this->_cnabSantander->modificaLinhasConsolidacaoSantander($tipoRegistro, $linha, $this->_valorTotalPagamento);
                break;
            case CNABBrasil::CODIGO_BANCO_DO_BRASIL:
                $linha = $this->_cnabBrasil->modificaLinhasConsolidacaoBrasil($tipoRegistro, $linha, $this->_valorTotalPagamento);
                break;
        }

        return $linha;
    }


    /**
     * Utilizada apenas no retrocesso. Modifica a sequencia dos registros 0 que pertencem ao mesmo envio.
     * Mesmo envio = mesma competencia e mesmo banco. Cada 0 representa um novo arquivo.
     *
     * @return void
     *
     * @author thalesvalias - 13/08/2012
     */
    private function _modificaSequenciaArquivo() {
        $this->_cnabMagnetico->modificaSequenciaArquivoRemessa($this->_mesCompetencia, $this->_nrOrg, $this->_cdBancoFormulario);
    }


    /**
     * Retrocede os registros da Org. no mes de competencia.
     *
     * @return void
     *
     * @author thalesvalias - 31/07/2012
     */
    public function retroceder(){
        $this->_cnabMagnetico->retroceder($this->_nrOrg, $this->_mesCompetencia, $this->_cdBancoFormulario, $this->_nrTpMovimentoFormulario);
        // Modifica a sequencia do arquivo apos o retrocesso, a fim de reorganizar.
//        $this->_modificaSequenciaArquivo();
    }


    private function _definePagamentoCnab(){

        // Carrega a lsita de itens de calculo de folha do vinculo corrente.
        $this->_itemCalculoFolha->retornaItensPorVinculo($this->_nrOrg, $this->_vinculo->getNrvinculom(), $this->_nrCalculoFolha);
        // Retorna o item de calculo de folha do evento de pagamento.
        $itemCalculoPagamento = $this->_itemCalculoFolha->procuraEvento($this->_eventoPagamento);

        // Define valor e a data de pagamento.
        if ($itemCalculoPagamento !== -1) {
            $this->_vinculo->valorPagamento = (float)$this->_itemCalculoFolha->retornaValor($itemCalculoPagamento);
//            $this->_vinculo->dataPagamento = $this->_itemCalculoFolha->retornaDtocorrencia($itemCalculoPagamento);
            $this->_vinculo->dataPagamento = $this->_dtPagamento;
            $this->_valorTotalPagamento += $this->_vinculo->valorPagamento;
        } else {
            $this->_vinculo->valorPagamento = FPARegistro::CARACTER_ZERO;
            $this->_vinculo->dataPagamento = FPARegistro::CARACTER_ZERO;
        }
    }


    public function getValorTotalPagamento(){
        return $this->_valorTotalPagamento;
    }


}