import {
  Metadata, Loader, IEventParam, Http, Config, IDictionary, Datasource,
  dayjs as moment, I18n,
} from '@zeedhi/core';
import {
  Select, Grid, TreeGridEditable, Form, IComponentRender, ModalService,
  Modal, GridEditable, DialogService, Button, Tabs, Date as ZdDate, Number, Radio,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pla01300PratoController {
  public crudController = Loader.getInstance('CrudController');
  
  private utilities = Loader.getInstance('Utilities');

  public gridEditController = Loader.getInstance('GridEditableController');

  public appController = Loader.getInstance('AppController');
  
  public receita = Loader.getInstance('ReceitaController');
  
  public consultaNutrientes = Loader.getInstance('Pla01305ConsultaNutrientesController');
  
  public modoPreparo = Loader.getInstance('Pla01300ModoPreparoController');

  public consultaCustos = Loader.getInstance('Pla01304ConsultaCustosController');
  
  public dadosReceita = Loader.getInstance('Pla01300DadosReceitaController');
  
  public modalAddPlate!: Modal;
  
  public modalEmissaoReceitas!: Modal;
  
  public modalGroupOperador!: Modal;
  
  public modalImport!: Modal;
  
  private recipeDest = '';

  public codeTree: IDictionary = {};

  public levelMax: any = '';

  public plateLocal: string = 'N';
  
  public plateLocalIqualPadr: string = 'N';
  
  public setNameTab: string = '';
  
  public cdGroupOperRepa: string = '';
  
  public operatorValid = false;
  
  public nrcorsafra: any = '';
  
  public nrcorinicsaf: any = '';
  
  public nrcorentrsaf: any = '';
  
  public project: string = 'PLA';
  
  public TCO: boolean = true;
  
  public inicContainer() {
    // trata o projeto
    // const project = Config.env.product.id;
    const project = localStorage.getItem('PRODUCT_ID');
    if (project === '4875') {
      this.project = 'PAE';
    } else {
      this.project = 'PLA';
    }
    
    // valida receitas TCO
    Http.post('/prato/returnParamTco', {}).then((result) => {
      const paramTco = result.data.dataset.paramTco[0];
      if (paramTco === 'N' || paramTco.IDREVRECPADQREGIAO === 'N') {
        this.TCO = false;
      }
    });
    
    this.validOperator();
  }
  
  public setFilterUnidadeDetail({ component }: IEventParam<Select>) {
    const values = {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA'),
      NMFILIAL: localStorage.getItem('NMFILIALLOGADA'),
    };
    component.datasource.data = [values];
    component.setValue(localStorage.getItem('CDFILIALLOGADA'));
  }
  
  /* public isVisibleIconPlateLocal({ row }: IDictionary) {
    return row.CDFILAUXPRAT !== 'P';
  } */

  public changePlate({ component }: IEventParam<Datasource>) {
    const cardPratos: any = Metadata.getInstance('cardTabs');
    const tabPratos: Tabs = Metadata.getInstance('tabPratos');
    const maskLevel = this.getLevelByMask(1);
    const nivelMax = maskLevel.length.toString();

    const plate = component.currentRow.CDPRATO;
    const levelPlate = component.currentRow.CDNVPRATO;
    
    if (plate) {
      if (levelPlate === nivelMax) {
        this.scrollTab();
        this.mountedDadosPlate();
        this.isVisibleTabTypeMenu();
        this.visibleRecipeStandardQualityAndRegion();
        cardPratos.isVisible = true;
        if (tabPratos.activeTab !== 0 && tabPratos.activeTab === 1) this.setfilterEmbalagem();
        else if (tabPratos.activeTab !== 0 && tabPratos.activeTab === 2) this.setfilterRecipeStandard();
        else if (tabPratos.activeTab !== 0 && tabPratos.activeTab === 3) this.setfilterRecipeTpCard();
        else if (tabPratos.activeTab !== 0) {
          this.setFilterUnidadeLocal();
          this.setFilterUnidadeService();
          this.setfilterRecipeRegion();
          this.setfilterRecipePadraoQualidade();
        }
        tabPratos.activeTabValue = 0;
      } else cardPratos.isVisible = false;
    } else cardPratos.isVisible = false;
  }
  
  public closeTabs() {
    const tabPratos: any = Metadata.getInstance('cardTabs');
    tabPratos.isVisible = false;
  }

  public scrollTab() {
    const element = document.getElementsByName('tabPratos')[0];

    setTimeout(() => {
      element.scrollIntoView();
    }, 1);
  }

  public setFilterBranch({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: 'OPERADOR_LOGADO',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterPlateTree({ component }: IEventParam<Select>) {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const buttonAdd = Metadata.getInstance<Button>('addTree');
    const buttonSeach = Metadata.getInstance<Button>('searchTreeGridPlate');
    if (component.value !== null) {
      const conditions = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: component.value.CDFILIAL,
        },
      ];
      
      plateTree.datasource.filter = conditions;
      plateTree.datasource.filter.limit = 1000;
      plateTree.buildTree();
      buttonAdd.disabled = false;
      buttonSeach.disabled = false;
      
      this.utilities.getParamPlaneja(component.value.CDFILIAL).then((paramplaneja: IDictionary) => {
        this.nrcorsafra = this.utilities.TColorToHex(paramplaneja?.NRCORSAFRA);
        this.nrcorinicsaf = this.utilities.TColorToHex(paramplaneja?.NRCORINICSAF);
        this.nrcorentrsaf = this.utilities.TColorToHex(paramplaneja?.NRCORENTRSAF);
      });
    } else {
      const conditions = [{}];
      plateTree.datasource.filter = conditions;
      plateTree.updateData([]);
      buttonAdd.disabled = true;
      buttonSeach.disabled = true;
    }
  }

  public addArvPlate() {
    const buttonAddForm = Metadata.getInstance<Button>('addTreeDadosPlate');
    const buttonAdd = Metadata.getInstance<Button>('addTree');
    buttonAddForm.disabled = true;
    buttonAdd.disabled = true;
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const form = Metadata.getInstance<Form>('formUnidade');
    const row = form.value;

    row.COLUMN_NAME = 'CDGROPERREPA';
    row.ALIAS_NAME = 'ALTRECEPADRAO';
    Http.post('/prato/returnParamsAddPlate', { requestType: 'Row', row }).then((callback) => {
      if (callback && callback.data.dataset && callback.data.dataset.permission && callback.data.dataset.paramgeral.length) {
        const result = callback.data.dataset.permission.result;
        const receitaRevisada = callback.data.dataset.paramgeral[0].IDRECREVICOP;
        this.plateLocal = callback.data.dataset.paramgeral[0].IDBLOPRATLOC;
        this.plateLocalIqualPadr = callback.data.dataset.paramgeral[0].IDBLOQPRADUP;

        if (result === 'NOPERMISSION' && receitaRevisada === 'S') {
          this.utilities.showAlert('Alert', 'pme::pratoLocalOpSemPermissao', 'error');
          buttonAdd.disabled = false;
          buttonAddForm.disabled = false;
        } else {
          const currentRow = plateTree.datasource.currentRow;
          this.generateCodePlate(currentRow, row);
        }
      } else {
        this.utilities.showAlert('Alert', 'pme::paramPlanejaSemConf', 'error');
        buttonAdd.disabled = false;
        buttonAddForm.disabled = false;
      }
    });
  }
  
  public generateCodePlate(currentRow: any, row: any) {
    const buttonAddForm = Metadata.getInstance<Button>('addTreeDadosPlate');
    const buttonAdd = Metadata.getInstance<Button>('addTree');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    let pratoPai = currentRow.CDPRATO === undefined ? '' : currentRow.CDPRATO;
    const filAux = currentRow.CDFILAUXPRAT === undefined ? 'P' : currentRow.CDFILAUXPRAT;
    let levelPlate = currentRow.CDNVPRATO === undefined ? 0 : currentRow.CDNVPRATO;
    const unity = row.UNIDADEFORM;
    const mask: string = (Config.masks.plateMask as PmeMask).pattern;
    const maskLevel = this.getLevelByMask(1);
    const nivelMax = maskLevel.length;

    if (levelPlate === nivelMax.toString()) {
      levelPlate -= 1;
      pratoPai = pratoPai.substr(0, maskLevel[levelPlate - 1].caracteres);
    }

    if ((levelPlate === 0) || (levelPlate === 1 && pratoPai === '')) {
      levelPlate = 0;
      pratoPai = 0;
    }

    const searchLevel = this.searchLevelOpen(currentRow, plateTree);
    
    if (searchLevel === 'PSP') this.utilities.showAlert('alert', 'pme::pratoPaiInexistente', 'error');
    else {
      if (searchLevel.length > 0) {
        pratoPai = ((searchLevel.split('_'))[0].substr(-(maskLevel[levelPlate].caracteres))).split('.').join('.');
        levelPlate = (searchLevel.split('_'))[1];
      }
      Http.post('/prato/searchNewCode', {
        pratoPai, filAux, levelPlate, unity, mask,
      }).then((newCode) => {
        const result = newCode.data.dataset.CODIGO[0].toString();
        let code;
        if (result === 0) code = result.padEnd(maskLevel[levelPlate].caracteres, '0');
        else code = result.padStart(maskLevel[levelPlate].caracteres, '0');
  
        if (code) this.openModalAddPlate(code);
        buttonAdd.disabled = false;
        buttonAddForm.disabled = false;
      });
    }
  }

  public searchLevelOpen(currentRow: any, plateTree: TreeGridEditable) {
    let pratoPaiNivelPai = '';
    let nivelAtual = 1;
    let prato;
    
    if (currentRow.CDNVPRATO > 1) nivelAtual = currentRow.CDNVPRATO;

    plateTree.datasource.data.forEach((dado: IDictionary) => {
      if (currentRow.CDPRATO) {
        if (dado.CDPRATO.split('.').join('') === currentRow.CDPRATO.split('.').join('')) {
          prato = dado;
        }
      }
    });

    if (!prato) prato = currentRow;
    
    if (prato.CDPRATONIVANTE === null && nivelAtual > 1) pratoPaiNivelPai = 'PSP';
    else {
      while (nivelAtual > 1) {
        const parentPlate = plateTree.datasource.currentRow.tree__parent;
        const parentPlateOpened = plateTree.datasource.currentRow.tree__parent.tree__opened;
        if (parentPlate) {
          if (!parentPlateOpened) {
            pratoPaiNivelPai = nivelAtual === 2 ? `${parentPlate.CDPRATO}_${parentPlate.CDNVPRATO}` : `${parentPlate.tree__parent.CDPRATO}_${parentPlate.tree__parent.CDNVPRATO}`;
          }
        }
        prato = parentPlate;
        nivelAtual -= 1;
      }
    }
    return pratoPaiNivelPai;
  }

  public getLevelByMask(firstLevel: any) {
    const mask: string = (Config.masks.plateMask as PmeMask).pattern;
    const splitMask = mask.split('.');
    const biggestLevel = splitMask.length;

    const level: any = [];
    let objLevel = {};

    for (let i = 0; i < biggestLevel; i += 1) {
      objLevel = {
        nivel: i + firstLevel,
        caracteres: i > 0 ? level[i - 1].caracteres + splitMask[i].length : splitMask[i].length,
      };

      level.push(objLevel);
    }
    return level;
  }

  public reloadTree() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const unidade = Metadata.getInstance<Form>('formUnidade');
    if (unidade.validate()) {
      plateTree.buildTree();
      if (plateTree.editing) this.cancelEditingTreeGrid();
      plateTree.selectedRows = [];
    } else plateTree.datasource.data = [];
  }

  public cancelEditingTreeGrid() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    plateTree.cancelEditedRows();
    const toolbarSlot = plateTree.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton') e.disabled = false;
      if (e.name === 'saveButton') e.disabled = false;
    });
    plateTree.buildTree();
  }

  // Funções Modal Add Plate

  private async createModalAddPlate() {
    this.modalAddPlate = await ModalService.createFromJson('planejamento/prato/modals/formAddPrato', true);
  }

  public closeModalAddPlate() {
    this.modalAddPlate.hide();
    
    const buttonAddForm = Metadata.getInstance<Button>('addTreeDadosPlate');
    const buttonAdd = Metadata.getInstance<Button>('addTree');
    
    buttonAdd.disabled = false;
    buttonAddForm.disabled = false;
  }
  
  public openModalAddPlate(code: any) {
    this.codeTree = code;
    if (!this.modalAddPlate) {
      this.createModalAddPlate().then(() => {
        this.modalAddPlate.show();
      });
    } else {
      this.modalAddPlate.show();
    }
  }

  public disabledFields() {
    const form = Metadata.getInstance<Form>('formAddPlate');
    const formRow = form.value;
    
    const maskLevel = this.getLevelByMask(1);
    const levelMax = maskLevel.length;
    formRow.CDNVPRATO = '';
    
    maskLevel.forEach((row: IDictionary) => {
      if (row.caracteres === formRow.CDPRATO.length) formRow.CDNVPRATO = row.nivel;
    });
    
    if (!formRow.CDNVPRATO) {
      this.utilities.showAlert('alert', 'pme::codigoPratoInvalido', 'error');
      return;
    }
    
    const tipo = form.getChildInstance<Select>('CDFILAUXPRAT');
    const un = form.getChildInstance<Select>('DSUNIDADE');
    const arredondTranp = form.getChildInstance<Number>('VRARREDTRANS');
    const preparo = form.getChildInstance<Select>('NMFORMAPREPARO');
    const tempo = form.getChildInstance<Number>('QTTEMPOPRAT');
    const frequencia = form.getChildInstance<Select>('IDFREQUENCIA');
    const status = form.getChildInstance<Radio>('IDATIVO');
    const dateInativacao = form.getChildInstance<ZdDate>('DTINATIVAUTO');
    
    if (tipo && un && arredondTranp && preparo && tempo && frequencia && status && dateInativacao) {
      if (formRow.CDNVPRATO === levelMax) {
        tipo.disabled = false;
        un.disabled = false;
        arredondTranp.disabled = false;
        preparo.disabled = false;
        tempo.disabled = false;
        frequencia.disabled = false;
        status.disabled = false;
        status.value = 'S';
        tipo.value = 'P';
        dateInativacao.disabled = false;
      } else {
        tipo.disabled = true;
        tipo.value = 'P';
        un.disabled = true;
        un.removeValidation('required');
        un.value = '';
        arredondTranp.disabled = true;
        arredondTranp.value = null;
        preparo.disabled = true;
        preparo.value = '';
        tempo.disabled = true;
        tempo.value = null;
        frequencia.disabled = true;
        frequencia.value = '';
        status.disabled = true;
        status.value = 'S';
        dateInativacao.disabled = true;
        dateInativacao.value = '';
      }
    }
  }
  
  /* public validTime() {
    const formModalPlate = Metadata.getInstance<Form>('formAddPlate');
    let time = formModalPlate.value.QTTEMPOPRAT;
    
    if (time < 1 && time !== null) {
      this.utilities.showAlert('alert', 'pme::valorTempoMaiorZero', 'warning');
      time = '';
      return false;
    }
    
    return time;
  }
  
  public validArredond() {
    const formModalPlate = Metadata.getInstance<Form>('formAddPlate');
    let arredondamento = formModalPlate.value.VRARREDTRANS;
    if (arredondamento <= 0 && arredondamento !== null) {
      this.utilities.showAlert('alert', 'pme::valorArredondMaiorZero', 'warning');
      arredondamento = '';
      return false;
    }
    
    return arredondamento;
  } */
  
  public savePlate() {
    const buttonSave = Metadata.getInstance<Button>('savePlateAdd');
    this.utilities.activeLoadingButton(buttonSave);
    
    const formModalPlate = Metadata.getInstance<Form>('formAddPlate');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const unidade = Metadata.getInstance<Form>('formUnidade').value.UNIDADEFORM.CDFILIAL;
    const currentRow = plateTree.datasource.currentRow;
    const rowForm = formModalPlate.value;
    const maskLevel = this.getLevelByMask(1);
    const levelMax = maskLevel.length.toString();
    const permissaoPadrao = true; // grupo de acesso
    
    maskLevel.forEach((row: IDictionary) => {
      if (row.caracteres === rowForm.CDPRATO.length) rowForm.CDNVPRATO = row.nivel;
    });
    
    if (rowForm.CDNVPRATO.toString() === levelMax) {
      const time = this.validTimeForm(rowForm);
      const arred = this.validArredondForm(rowForm);
      if (time === false || arred === false) {
        this.utilities.disableLoadingButton(buttonSave);
        return;
      }
      
      rowForm.QTTEMPOPRAT = time;
      rowForm.VRARREDTRANS = arred;
      rowForm.CDFILIAL = unidade;
    } else {
      rowForm.CDFILIAL = '';
    }
    
    if (formModalPlate.validate()) {
      if (rowForm.QTTEMPOPRAT !== 0 && rowForm.VRARREDTRANS !== 0) {
        if (rowForm.IDATIVO !== null) {
          if (currentRow.CDNVPRATO < levelMax - 1 && !permissaoPadrao) this.utilities.showAlert('alert', 'pme::semPermissaoAddPratoNvlComp', 'error');
          else {
            const codePlate = rowForm.CDPRATO;
            const level = rowForm.CDNVPRATO;
            if (!level) {
              this.utilities.showAlert('alert', 'pme::codigoPratoInvalido', 'error');
              this.utilities.disableLoadingButton(buttonSave);
              return;
            }
            
            let filAux = 'P';
            let CDPRATO = '';
    
            if (level > 1) CDPRATO = codePlate.substr(0, maskLevel[level - 2].caracteres);
            else CDPRATO = codePlate.substr(0, maskLevel[level - 1].caracteres);
          
            let paramPL = 'N';
            Http.post('/prato/returnPlate', { CDPRATO, filAux, paramPL }).then((result) => { // valid dad plate exist.
              const resultPlate = result.data.dataset.PRATO;
              if (resultPlate.CDPRATO === false && level > 1) this.utilities.showAlert('alert', 'pme::pratoPaiInexistente', 'error');
              else {
                if (!CDPRATO && level > 1) {
                  plateTree.datasource.data.forEach((row) => {
                    if (row.CDPRATO === CDPRATO) {
                      plateTree.datasource.currentRow = row;
                    }
                  });
                }
                
                CDPRATO = rowForm.CDPRATO;
                filAux = rowForm.CDFILAUXPRAT;
                paramPL = this.plateLocalIqualPadr;
                Http.post('/prato/returnPlate', { CDPRATO, filAux, paramPL }).then((codeEqual) => { // valid plate insert exist.
                  const resultCodeEqual = codeEqual.data.dataset.PRATO;
                  if (resultCodeEqual.CDPRATO) {
                    this.utilities.showAlert('alert', 'pme::pratoEncontradoArvore', 'error');
                  } else if (this.plateLocal === 'S') {
                    if (rowForm.CDFILAUXPRAT === 'P') {
                      this.saveRowPlate(level, levelMax, rowForm);
                    } else {
                      this.utilities.showAlert('alert', 'pme::opBloqAddPratoLocalMarcada', 'error');
                    }
                  } else this.saveRowPlate(level, levelMax, rowForm);
                });
              }
            }).finally(() => {
              this.utilities.disableLoadingButton(buttonSave);
            });
          }
        } else this.utilities.showAlert('alert', 'pme::statusObrig', 'error');
      }
    } else this.utilities.disableLoadingButton(buttonSave);
  }
  
  public calculoValidade(tempo: any, frequencia: any) {
    if (frequencia === 'H') return tempo;
    if (frequencia === 'D') return tempo * 24;
    if (frequencia === 'M') return tempo * 24 * 30;
    if (frequencia === 'A') return tempo * 24 * 30 * 12;
    return 0;
  }
  
  public saveRowPlate(level: any, levelMax: any, rowForm: any) {
    const buttonSave = Metadata.getInstance<Button>('savePlateAdd');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const form = Metadata.getInstance<Form>('formUnidade');
    const maskLevel = this.getLevelByMask(1);
    if (level < levelMax) {
      rowForm.CDFILAUXPRAT = 'P';
      rowForm.CDFILIAL = '';
      rowForm.SGUNIDADE = '';
      rowForm.VRARREDTRANS = '';
      rowForm.NRFORMAPREPARO = '';
      rowForm.QTTEMPOPRAT = '';
      rowForm.IDFREQUENCIA = '';
      rowForm.__is_new = true;
    } else {
      rowForm.CDFILIAL = form.value.UNIDADEFORM.CDFILIAL;
      if (rowForm.CDFILAUXPRAT === 'L') {
        rowForm.CDFILAUXPRAT = form.value.UNIDADEFORM.CDFILIAL;
      }
    }
    
    if (rowForm.QTTEMPOPRAT === undefined) rowForm.QTTEMPOPRAT = null;
    if (rowForm.VRARREDTRANS === undefined) rowForm.VRARREDTRANS = null;
    rowForm.VRVALIDADE = rowForm.QTTEMPOPRAT ? this.calculoValidade(rowForm.QTTEMPOPRAT, rowForm.IDFREQUENCIA) : 0;
    rowForm.CDPRATO = rowForm.CDPRATO.split('.').join('');
    rowForm.IDATIVO = rowForm.IDATIVO === null ? 'S' : rowForm.IDATIVO;
    maskLevel.forEach((row: IDictionary) => {
      if (row.caracteres === rowForm.CDPRATO.length) rowForm.CDNVPRATO = row.nivel;
    });
    
    Http.post('/prato/savePlate', { requestType: 'Row', rowForm }).then(() => {
      this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      plateTree.buildTree();
    }).catch((error: string) => {
      if (error.indexOf('ORA-00001') >= 0) this.utilities.showAlert('alert', 'pme::impossivelAdicionar', 'error');
      else this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      buttonSave.disabled = false;
      this.closeModalAddPlate();
    });
  }
  
  public updatePlate() {
    const buttonUpdate = Metadata.getInstance<Button>('saveButton');
    buttonUpdate.disabled = true;
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const editedRows = plateTree.getEditedRows();

    Http.post('/prato/updatePlate', { editedRows }).then(() => {
      this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      const toolbarSlot = plateTree.toolbarSlot;
      toolbarSlot.forEach((e: IComponentRender) => {
        if (e.name === 'cancelButton') e.disabled = true;
        if (e.name === 'saveButton') e.disabled = true;
      });
      
      plateTree.buildTree();
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    }).finally(() => {
      plateTree.cancelEditedRows();
    });
  }
  
  public confirmDeletePlate({ component }: IEventParam<Button>) {
    const nameButton = component.name;
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    let editedRows: any = [];
    
    if (nameButton === 'excluiPlateCompl') {
      const formDadosPlate = Metadata.getInstance<Form>('formDadosPrato');
      const rowDadosPlate = formDadosPlate.value;
      
      editedRows[0] = {
        CDFILAUXPRAT: rowDadosPlate.CDFILAUXPRAT,
        CDFILIAL: rowDadosPlate.CDFILIAL,
        CDNVPRATO: rowDadosPlate.CDNVPRATO,
        CDPRATO: rowDadosPlate.CDPRATO,
      };
    } else {
      editedRows = plateTree.selectedRows;
    }
    
    if (editedRows.length === 0) {
      this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    } else {
      DialogService.show({
        name: nameButton,
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteTree(plateTree, editedRows) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public deleteTree(plateTree: TreeGridEditable, editedRows: any) {
    const buttonYes = Metadata.getInstance<Button>('yes');
    buttonYes.disabled = true;

    Http.post('/prato/deletePlate', { editedRows }).then(() => {
      this.utilities.showAlert('Alert', 'pme::excluidoComSucesso', 'success');
      plateTree.selectedRows = [];
      plateTree.buildTree();
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
      if (error.includes('ORA-02292')) this.utilities.showAlert('alert', 'pme::ERROR_REGISTROFILHO', 'error');
    }).finally(() => {
      DialogService.hide();
      buttonYes.disabled = false;
    });
  }
  
  public isDetailsActionVisible({ row }: IDictionary) {
    return row.CDFILAUXPRAT !== 'P';
  }

  // Funções da aba Embalagem

  public setfilterEmbalagem() {
    const grid: GridEditable = Metadata.getInstance('gridEmbalagem');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: currentRow.CDPRATO,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: currentRow.CDFILAUXPRAT,
      },
    ];
    grid.datasource.filter = conditions;
    grid.datasource.get();
  }
  
  public setValueQty({ component }: IEventParam<Select>) {
    const grid: GridEditable = Metadata.getInstance('gridEmbalagem');
    const qtContainer = component.datasource.currentRow.QTCONTAINER;

    if (qtContainer) grid.datasource.currentRow.QTPRATCONTAINER = qtContainer;
  }

  public saveEmbalagem() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const grid: GridEditable = Metadata.getInstance('gridEmbalagem');
    const editedRows = grid.getEditedRows();
    if (editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        if (row.__is_new) {
          row.CDPRATO = plateTree.datasource.currentRow.CDPRATO;
          row.CDFILAUXPRAT = plateTree.datasource.currentRow.CDFILAUXPRAT;
          row.CDCONTAINER = row.NMCONTAINER.CDCONTAINER;
          row.NMCONTAINER = row.NMCONTAINER.NMCONTAINER;
          row.NRORG = '';
        }
      });

      if (editedRows[0].IDPRATCONTPRI === 'S') {
        let update;
        grid.datasource.data.forEach((item: IDictionary) => {
          if (!item.__is_new && item.IDPRATCONTPRI === 'S' && item.CDCONTAINER !== editedRows[0].CDCONTAINER) {
            item.__is_new = false;
            update = item;
          }
        });

        if (update) {
          Http.post('/prato/updatePackaging', { requestType: 'Row', update }).then(() => {
            this.callSavePack(editedRows, grid);
          });
        } else this.callSavePack(editedRows, grid);
      } else this.callSavePack(editedRows, grid);
    }
  }
  
  public deleteEmbalagem({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      const itemPrinc = checkedRows.filter((item) => item.IDPRATCONTPRI === 'S');
      if (itemPrinc.length > 0) this.utilities.showAlert('alert', 'pme::algumItemPrincipal', 'error');
      else {
        DialogService.show({
          name: 'excluiEmbPrat',
          type: 'warning',
          title: 'pme::atencao',
          text: 'pme::desejaExcluirRegistros',
          buttons: [
            {
              name: 'yes',
              component: 'ZdButton',
              label: 'pme::sim',
              events: { click: () => this.gridEditController.deleteCheckedRows(component) },
              outline: true,
            },
            {
              name: 'no',
              component: 'ZdButton',
              label: 'pme::nao',
              events: { click: () => DialogService.hide() },
            },
          ],
        });
      }
    }
  }

  public callSavePack(editedRows: any, grid: GridEditable) {
    this.crudController.saveRows(editedRows, 'pratocontainer', grid).then(() => {
      this.utilities.showAlert('Alert', 'pme::salvoComSucesso', 'success');
      this.gridEditController.defaultState(grid);
      // this.gridEditController.reload({ grid });
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }
  
  public addEmbalagem({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const newId = '';
    const uniqueKey = grid.datasource.uniqueKey;
    
    if (!this.gridEditController.newRow) {
      grid.addNewRow({ [uniqueKey]: newId, __is_new: true, IDPRATCONTPRI: 'S' }, 'start');
      this.gridEditController.newRow = true;
      if (!grid.editing) {
        grid.editing = true;
        if (!this.gridEditController.isInlineEditing) this.gridEditController.onInlineEdit({ component: grid });
      }
      
      /* Coloca foco na primeira coluna do grid. Ficar atento a coluna invisível */
      const columnName = grid?.columns[0]?.name;
      setTimeout(() => {
        (grid.getComponent(newId, columnName) as IDictionary).setFocus();
      }, 100);
    }
  }
  
  // SetForm Dados do Prato
  
  public mountedDadosPlate() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    const formDadosPlate = Metadata.getInstance<Form>('formDadosPrato');
    const rowDadosPlate = formDadosPlate.value;
    const dsUnid = Metadata.getInstance<Select>('DSUNIDADEDP');
    const prepare = Metadata.getInstance<Select>('NMFORMAPREPARODP');
    
    const cancel = Metadata.getInstance<Button>('cancelButtonDadosPlate');
    const save = Metadata.getInstance<Button>('saveButtonDadosPlate');
    // rowDadosPlate.CDFILIAL = currentRow.CDFILIAL;
    const row = {
      CDFILIAL: currentRow.CDFILIAL,
      CDFILAUXPRAT: currentRow.CDFILAUXPRAT,
      CDPRATO: currentRow.CDPRATO,
    };
    
    Http.post('/prato/resultDadosPlate', { requestType: 'Row', row }).then((result) => {
      const retorno = result.data.dataset.result[0];
      if (retorno.CDFILAUXPRAT === retorno.CDFILIAL) retorno.CDFILAUXPRAT = 'L';
      if (retorno.DTINATIVAUTO !== null) retorno.DTINATIVAUTO = moment(retorno.DTINATIVAUTO, 'DD/MM/YYYY');
      rowDadosPlate.CDFILAUXPRAT = retorno.CDFILAUXPRAT;
      rowDadosPlate.CDPRATO = retorno.CDPRATO;
      rowDadosPlate.NMPRATO = retorno.NMPRATO;
      rowDadosPlate.CDNVPRATO = retorno.CDNVPRATO;
      rowDadosPlate.CDFILIAL = retorno.CDFILIAL;
      rowDadosPlate.DTINATIVAUTO = retorno.DTINATIVAUTO;
      rowDadosPlate.IDATIVO = retorno.IDATIVO;
      rowDadosPlate.IDFREQUENCIA = retorno.IDFREQUENCIA;
      rowDadosPlate.QTTEMPOPRAT = retorno.QTTEMPOPRAT;
      rowDadosPlate.VRARREDTRANS = retorno.VRARREDTRANS;
      const dsUnidade = {
        SGUNIDADE: retorno.SGUNIDADE,
        DSUNIDADE: retorno.DSUNIDADE,
      };
      dsUnid.datasource.data = [dsUnidade];
      dsUnid.setValue(retorno.SGUNIDADE);
      if (retorno.NRFORMAPREPARO) {
        const formaPreparo = {
          NRFORMAPREPARO: retorno.NRFORMAPREPARO,
          NMFORMAPREPARO: retorno.NMFORMAPREPARO,
        };
        prepare.datasource.data = [formaPreparo];
        prepare.setValue(retorno.NRFORMAPREPARO);
      } else prepare.setValue('');
    });
    
    cancel.disabled = true;
    save.disabled = true;
  }
  
  public disableButtonsDadosPlate() {
    const cancel = Metadata.getInstance<Button>('cancelButtonDadosPlate');
    const save = Metadata.getInstance<Button>('saveButtonDadosPlate');
    cancel.disabled = false;
    save.disabled = false;
  }
  
  public cancelForm() {
    const cancel = Metadata.getInstance<Button>('cancelButtonDadosPlate');
    const save = Metadata.getInstance<Button>('saveButtonDadosPlate');
    cancel.disabled = true;
    save.disabled = true;
    this.mountedDadosPlate();
  }
  
  public updateDadosPlate() {
    const buttonUpdate = Metadata.getInstance<Button>('saveButtonDadosPlate');
    buttonUpdate.disabled = true;
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const formDadosPlate = Metadata.getInstance<Form>('formDadosPrato');
    const rowDadosPlate = formDadosPlate.value;
    rowDadosPlate.CDFILAUXPRAT_UPDATE = plateTree.datasource.currentRow.CDFILAUXPRAT;
    
    const time = this.validTimeForm(rowDadosPlate);
    const arred = this.validArredondForm(rowDadosPlate);
    if (time === false || arred === false) {
      return;
    }
   
    rowDadosPlate.QTTEMPOPRAT = time;
    rowDadosPlate.VRARREDTRANS = arred;
    
    if (rowDadosPlate.QTTEMPOPRAT !== 0 && rowDadosPlate.VRARREDTRANS !== 0) {
      if (rowDadosPlate.CDPRATO !== null) {
        Http.post('/prato/updateDadosPlate', { rowDadosPlate }).then(() => {
          this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
          if (plateTree.datasource.currentRow.NMPRATO !== rowDadosPlate.NMPRATO) {
            // this.mountedDadosPlate();
            plateTree.buildTree();
          }
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        }).finally(() => {
          // this.mountedDadosPlate();
          this.cancelForm();
          plateTree.cancelEditedRows();
        });
      }
    }
  }
  
  public validTimeForm(rowDadosPlate: any) {
    let time = rowDadosPlate.QTTEMPOPRAT;
    
    if (time < 1 && time !== null) {
      this.utilities.showAlert('alert', 'pme::valorTempoMaiorZero', 'warning');
      time = '';
      return false;
    }
    
    return time;
  }
  
  public validArredondForm(rowDadosPlate: any) {
    let arredondamento = rowDadosPlate.VRARREDTRANS;
    if (arredondamento <= 0 && arredondamento !== null) {
      this.utilities.showAlert('alert', 'pme::valorArredondMaiorZero', 'warning');
      arredondamento = '';
      return false;
    }
    
    return arredondamento;
  }
  
  // Receitas
  
  public setFilterUnidadeLocal() {
    const formUnidade = Metadata.getInstance<Form>('formUnidade');
    const unidLocal = Metadata.getInstance<Select>('BRANCH_LOCAL');
    const values = {
      CDFILIAL: formUnidade.value.UNIDADEFORM.CDFILIAL,
      NMFILIAL: formUnidade.value.UNIDADEFORM.NMFILIAL,
    };
    unidLocal.datasource.data = [values];
    unidLocal.setValue(formUnidade.value.UNIDADEFORM.CDFILIAL);
  }
  
  public setFilterUnidadeService() {
    const formUnidade = Metadata.getInstance<Form>('formUnidade');
    const unidLService = Metadata.getInstance<Select>('BRANCH_SERVICE');
    const values = {
      CDFILIAL: formUnidade.value.UNIDADEFORM.CDFILIAL,
      NMFILIAL: formUnidade.value.UNIDADEFORM.NMFILIAL,
    };
    unidLService.datasource.data = [values];
    unidLService.setValue(formUnidade.value.UNIDADEFORM.CDFILIAL);
  }
  
  // Receita Padrão
  
  public setfilterRecipeStandard() {
    const grid: GridEditable = Metadata.getInstance('RECIPE_STANDARD');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: currentRow.CDPRATO,
      },
      {
        name: 'CDFILAUXPRAT',
        operator: '=',
        value: currentRow.CDFILAUXPRAT,
      },
    ];
    grid.datasource.filter = conditions;
    grid.datasource.get().then(() => {
      // this.loadStandardRecipeData(conditions);
      const addButton: Button = grid.getChildInstance('addButton');
      addButton.disabled = false;
      // this.enabledTabs();
    });
  }
  
  public saveRecipeStandard({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    const form = Metadata.getInstance<Form>('formRecipeStandard');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    
    if (form.validate()) {
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;

      rows.forEach((e: IDictionary) => {
        e.CDPRATO = currentRow.CDPRATO;
        e.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
        // e.SGUNIDADE = e.SGUNIDPROD;
        e.QTLIMSEGPADR = e.QTLIMSEGPADR || null;
        e.QTCOMPLPRODPADR = e.QTCOMPLPRODPADR || 0;
        e.CDAPRERECE = e.CDAPRERECE || null;
        if (e.__is_new) e.CDPRODUTO = e.CDPRODUTOAUX;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPPR && !this.utilities.timeIsValid(e.HRTEMPPREPPR)) validHour = false;
        e.HRTEMPPREPPR = this.utilities.formatHour(e.HRTEMPPREPPR, false);
      });

      if (this.receita.saveIsValid(validHour, ctrlEstoq, active)) this.receita.callSave('/saveRecipeStandard', rows, grid);
    }
  }
  
  public getFilterProd() {
    const unidade = Metadata.getInstance<Select>('UNIDADEFORM');
    return [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value.CDFILIAL,
      },
    ];
  }
  
  // Receita Local
  
  public setFilterRecipeLocal() {
    const tab = this.setNameTab;
    if (document.getElementsByName('RECIPE_LOCAL').length > 0) {
      const form = Metadata.getInstance<Form>('FORM_LOCAL');
      const unidade = form.value.BRANCH_LOCAL?.CDFILIAL;
      const tabLocal = Metadata.getInstance<Tabs>('subTabReceitaLocal');
      
      let grid: any;
      if (tab === 'subTabReceitaLocal') {
        if (tabLocal.activeTab === 0) grid = Metadata.getInstance<GridEditable>('RECIPE_LOCAL');
        else if (tabLocal.activeTab === 2) grid = Metadata.getInstance<Grid>('CONSULTACUSTOS-Local');
        else if (tabLocal.activeTab === 3) grid = Metadata.getInstance<Grid>('CONSULTANUTRIENTES-Local');
      }
      
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      const currentRow = plateTree.datasource.currentRow;
      
      if (tab === 'subTabReceitaLocal') {
        const tabs = Metadata.getInstance<Tabs>('subTabReceitaLocal');
        if (tabs.activeTab === 0) {
          const conditions = [
            {
              name: 'CDPRATO',
              operator: '=',
              value: currentRow.CDPRATO,
            },
            {
              name: 'CDFILIAL',
              operator: '=',
              value: unidade,
            },
            {
              name: 'CDFILAUXPRAT',
              operator: '=',
              value: currentRow.CDFILAUXPRAT,
            },
          ];
          grid.datasource.filter = conditions;
        }
        
        if (!unidade) {
          if (tabLocal.activeTab === 0 || tabLocal.activeTab === 2 || tabLocal.activeTab === 3) {
            const addButton: Button = grid.getChildInstance('addButton');
            grid.datasource.data = [];
            addButton.disabled = true;
          } else if (tabLocal.activeTab === 1) this.dadosReceita.setValuesRecipe();
        } else if (unidade) {
          if (tabLocal.activeTab === 0 || tabLocal.activeTab === 2 || tabLocal.activeTab === 3) {
            grid.datasource.get().then(() => {
              const addButton: Button = grid.getChildInstance('addButton');
              addButton.disabled = false;
              if (tabs.activeTab === 2) this.consultaCustos.filterPlatesGrid();
              else if (tabs.activeTab === 3) this.consultaNutrientes.filterNutrientesGrid();
            });
          } else if (tabs.activeTab === 1) this.dadosReceita.setValuesRecipe();
        }
      }
    }
  }
  
  public saveRecipeLocal({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    const form = Metadata.getInstance<Form>('formRecipeLocal');
    const formUnidLocal = Metadata.getInstance<Form>('FORM_LOCAL');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    
    if (form.validate()) {
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;

      rows.forEach((e: IDictionary) => {
        e.CDFILIAL = formUnidLocal.value.BRANCH_LOCAL.CDFILIAL;
        e.CDPRATO = currentRow.CDPRATO;
        e.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
        // e.SGUNIDADE = e.SGUNIDPROD;
        e.QTLIMSEGFILI = e.QTLIMSEGFILI || null;
        e.QTCOMPLPRODFILI = e.QTCOMPLPRODFILI || 0;
        e.CDAPRERECE = e.CDAPRERECE || null;
        if (e.__is_new) e.CDPRODUTO = e.CDPRODUTOAUX;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPLO && !this.utilities.timeIsValid(e.HRTEMPPREPLO)) validHour = false;
        e.HRTEMPPREPLO = this.utilities.formatHour(e.HRTEMPPREPLO, false);
      });
      
      if (this.receita.saveIsValid(validHour, ctrlEstoq, active)) this.receita.callSave('/saveRecipeLocal', rows, grid);
    }
  }
  
  // Receita Serviço
  
  public changeUnityRecipeServ() {
    const tab = this.setNameTab;
    let grid: any;
    if (tab === 'subTabReceitaServico') {
      const tabServico = Metadata.getInstance<Tabs>('subTabReceitaServico');
      if (tabServico.activeTab === 0) grid = Metadata.getInstance<GridEditable>('RECIPE_SERVICE');
      else if (tabServico.activeTab === 2) grid = Metadata.getInstance<Grid>('CONSULTACUSTOS-Servico');
      else if (tabServico.activeTab === 3) grid = Metadata.getInstance<Grid>('CONSULTANUTRIENTES-Servico');
    }
    const serv = Metadata.getInstance<Select>('SERVICE');
    serv.setValue('');
    grid.datasource.data = [];
  }
  
  public setFilterRecipeService() {
    const tab = this.setNameTab;
    const grid: GridEditable = Metadata.getInstance('RECIPE_SERVICE');
    const form = Metadata.getInstance<Form>('FORM_SERVICE');
    const unidade = form.value.BRANCH_SERVICE?.CDFILIAL;
    const servico = form.value.SERVICE?.CDSERVFILI;
    if (document.getElementsByName('RECIPE_SERVICE').length > 0) {
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      const currentRow = plateTree.datasource.currentRow;
      if (form.validate()) {
        const conditions = [
          {
            name: 'CDPRATO',
            operator: '=',
            value: currentRow.CDPRATO,
          },
          {
            name: 'CDFILIAL',
            operator: '=',
            value: unidade,
          },
          {
            name: 'CDSERVFILI',
            operator: '=',
            value: servico,
          },
          {
            name: 'CDFILAUXPRAT',
            operator: '=',
            value: currentRow.CDFILAUXPRAT,
          },
        ];
        
        grid.datasource.filter = conditions;
        
        const addButton: Button = grid.getChildInstance('addButton');
        if (!unidade || !servico) {
          grid.datasource.data = [];
          addButton.disabled = true;
        } else {
          grid.datasource.get().then(() => {
            addButton.disabled = false;
            if (tab === 'subTabReceitaServico') {
              const tabs = Metadata.getInstance<Tabs>('subTabReceitaServico');
              if (tabs.activeTab === 1) this.dadosReceita.setValuesRecipe();
              else if (tabs.activeTab === 2) this.consultaCustos.filterPlatesGrid();
              else if (tabs.activeTab === 3) this.consultaNutrientes.filterNutrientesGrid();
            }
          });
        }
      }
    }
    const addButton: Button = grid.getChildInstance('addButton');
    if (!unidade || !servico) {
      grid.datasource.data = [];
      addButton.disabled = true;
    }
  }
  
  public saveRecipeService({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    const form = Metadata.getInstance<Form>('formRecipeService');
    const formService = Metadata.getInstance<Form>('FORM_SERVICE');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    
    if (form.validate()) {
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;

      rows.forEach((e: IDictionary) => {
        e.CDFILIAL = formService.value.BRANCH_SERVICE.CDFILIAL;
        e.CDSERVFILI = formService.value.SERVICE.CDSERVFILI;
        e.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
        e.CDPRATO = currentRow.CDPRATO;
        // e.SGUNIDADE = e.SGUNIDADE;
        e.QTLIMSEGSERV = e.QTLIMSEGSERV || null;
        e.QTCOMPLPRODSERV = e.QTCOMPLPRODSERV || 0;
        e.CDAPRERECE = e.CDAPRERECE || null;
        if (e.__is_new) e.CDPRODUTO = e.CDPRODUTOAUX;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPSE && !this.utilities.timeIsValid(e.HRTEMPPREPSE)) validHour = false;
        e.HRTEMPPREPSE = this.utilities.formatHour(e.HRTEMPPREPSE, false);
      });
      
      if (this.receita.saveIsValid(validHour, ctrlEstoq, active)) this.receita.callSave('/saveRecipeService', rows, grid);
    }
  }
  
  //  Receita Tipo de Cardápio
  
  public isVisibleTabTypeMenu() {
    const tabPlate = Metadata.getInstance<Tabs>('tabPratos');
    const tabTypeMenu = tabPlate.tabs.find((column: IDictionary) => column.name === 'tabReceitaTipoCardapio');
    if (tabTypeMenu) {
      if (this.project === 'PAE') tabTypeMenu.isVisible = true;
      else tabTypeMenu.isVisible = false;
    }
  }
  
  public setfilterRecipeTpCard() {
    const tab = this.setNameTab;
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    const tabTpCard = Metadata.getInstance<Tabs>('subTabReceitaTipoCard');
    
    if (document.getElementsByName('RECIPE_TIPOCARD').length > 0) {
      let grid: any;
      if (tab === 'subTabReceitaTipoCard') {
        if (tabTpCard.activeTab === 0) grid = Metadata.getInstance<GridEditable>('RECIPE_TIPOCARD');
        else if (tabTpCard.activeTab === 2) grid = Metadata.getInstance<Grid>('CONSULTACUSTOS-TipoCard');
        else if (tabTpCard.activeTab === 3) grid = Metadata.getInstance<Grid>('CONSULTANUTRIENTES-TipoCard');
      }
      
      const formTpCard = Metadata.getInstance<Form>('FORM_TIPOCARD');
      const tpCard = formTpCard.value.TIPOCARD?.CDTIPOCARD;
      
      if (tab === 'subTabReceitaTipoCard') {
        const tabs = Metadata.getInstance<Tabs>('subTabReceitaTipoCard');
        
        if (tabs.activeTab === 0) {
          const conditions = [
            {
              name: 'P_CDPRATO',
              operator: '=',
              value: currentRow.CDPRATO,
            },
            {
              name: 'P_CDFILAUXPRAT',
              operator: '=',
              value: currentRow.CDFILAUXPRAT,
            },
            {
              name: 'P_CDFILCENTPME',
              operator: '=',
              value: this.appController.getCdCentral(),
            },
            {
              name: 'P_CDTIPOCARD',
              operator: '=',
              value: tpCard,
            },
          ];
      
          grid.datasource.filter = conditions;
        }
        if (!tpCard) {
          if (tabTpCard.activeTab === 0 || tabTpCard.activeTab === 2 || tabTpCard.activeTab === 3) {
            const addButton: Button = grid.getChildInstance('addButton');
            grid.datasource.data = [];
            addButton.disabled = true;
          } else if (tabTpCard.activeTab === 1) this.dadosReceita.setValuesRecipe();
        } else if (tpCard) {
          if (tabTpCard.activeTab === 0 || tabTpCard.activeTab === 2 || tabTpCard.activeTab === 3) {
            const addButton: Button = grid.getChildInstance('addButton');
            grid.datasource.get().then(() => {
              addButton.disabled = false;
              if (tabs.activeTab === 2) this.consultaCustos.filterPlatesGrid();
              else if (tabs.activeTab === 3) this.consultaNutrientes.filterNutrientesGrid();
            });
          } else if (tabs.activeTab === 1) this.dadosReceita.setValuesRecipe();
        }
      }
    }
  }
  
  public saveRecipeTpCard({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    const form = Metadata.getInstance<Form>('formRecipeTpcard');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    const formTpCard = Metadata.getInstance<Form>('FORM_TIPOCARD');
    
    if (form.validate()) {
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;

      rows.forEach((e: IDictionary) => {
        e.CDTIPOCARD = formTpCard.value.TIPOCARD.CDTIPOCARD;
        e.CDPRATO = currentRow.CDPRATO;
        e.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
        // e.SGUNIDADE = e.SGUNIDPROD;
        e.CDAPRERECE = e.CDAPRERECE || null;
        if (e.__is_new) e.CDPRODUTO = e.CDPRODUTOAUX;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPPRTP && !this.utilities.timeIsValid(e.HRTEMPPREPPRTP)) validHour = false;
        e.HRTEMPPREPPRTP = this.utilities.formatHour(e.HRTEMPPREPPRTP, false);
        e.CDFILCENTPME = this.appController.getCdCentral();
      });

      if (this.receita.saveIsValid(validHour, ctrlEstoq, active)) this.receita.callSave('/saveRecipeTpCard', rows, grid);
    }
  }
  
  // Receita Padrão de Qualidade e Região
  
  public visibleRecipeStandardQualityAndRegion() {
    const tabPlate = Metadata.getInstance<Tabs>('tabPratos');
    const tabStandardQuality = tabPlate.tabs.find((column: IDictionary) => column.name === 'tabReceitaPadraoQualidade');
    const tabRegion = tabPlate.tabs.find((column: IDictionary) => column.name === 'tabReceitaRegiao');
    if (tabStandardQuality && tabRegion) {
      if (this.TCO) {
        tabStandardQuality.isVisible = true;
        tabRegion.isVisible = true;
      } else {
        tabStandardQuality.isVisible = false;
        tabRegion.isVisible = false;
      }
    }
  }
  
  public setfilterRecipeRegion() {
    if (document.getElementsByName('RECIPE_REGION').length > 0) {
      const grid: GridEditable = Metadata.getInstance('RECIPE_REGION');
      const form = Metadata.getInstance<Form>('FORM_REGIAO');
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      const currentRow = plateTree.datasource.currentRow;
      if (form.validate()) {
        const padraQual = form.value.NMPADRAOQUALIDADE.NRPADRAOQUALIDADE;
        const regiao = form.value.NMREGIAO.CDREGIAO;
        
        const conditions = [
          {
            name: 'CDPRATO',
            operator: '=',
            value: currentRow.CDPRATO,
          },
          {
            name: 'CDFILAUXPRAT',
            operator: '=',
            value: currentRow.CDFILAUXPRAT,
          },
          {
            name: 'NRPADRAOQUALIDADE',
            operator: '=',
            value: padraQual,
          },
          {
            name: 'CDREGIAO',
            operator: '=',
            value: regiao,
          },
        ];
        grid.datasource.filter = conditions;
        
        const addButton: Button = grid.getChildInstance('addButton');
        
        // validar
        if (!padraQual || !regiao) {
          addButton.disabled = true;
        } else {
          grid.datasource.get().then(() => {
            addButton.disabled = false;
          });
        }
      }
    }
  }
  
  public saveRecipeRegion({ component }: IDictionary) {
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    const form = Metadata.getInstance<Form>('formRecipeRegion');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    const formRegiao = Metadata.getInstance<Form>('FORM_REGIAO');
    
    if (form.validate()) {
      const rows: IDictionary = grid.datasource.data.filter((e: IDictionary) => e.EDITED);
      let ctrlEstoq = true;
      let active = true;
      let validHour = true;

      rows.forEach((e: IDictionary) => {
        e.NRRECEREGIAO = '';
        e.NRPADRAOQUALIDADE = formRegiao.value.NMPADRAOQUALIDADE.NRPADRAOQUALIDADE;
        e.CDPADRAOQUALIDADE = formRegiao.value.NMPADRAOQUALIDADE.CDPADRAOQUALIDADE;
        e.CDREGIAO = formRegiao.value.NMREGIAO.CDREGIAO;
        e.CDPRATO = currentRow.CDPRATO;
        e.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
        // e.SGUNIDADE = e.SGUNIDPROD;
        e.CDAPRERECE = e.CDAPRERECE || null;
        if (e.__is_new) e.CDPRODUTO = e.CDPRODUTOAUX;
        if (e.__is_new && e.IDCNTRESTOQ !== 'S') ctrlEstoq = false;
        if (e.__is_new && e.IDPRODATIV !== 'A') active = false;
        if (e.HRTEMPPREPPRTP && !this.utilities.timeIsValid(e.HRTEMPPREPPRTP)) validHour = false;
        e.HRTEMPPREPPRTP = this.utilities.formatHour(e.HRTEMPPREPPRTP, false);
        e.CDFILCENTPME = this.appController.getCdCentral();
      });

      if (this.receita.saveIsValid(validHour, ctrlEstoq, active)) this.receita.callSave('/saveRecipeRegion', rows, grid);
    }
  }
  
  public setfilterRecipePadraoQualidade() {
    if (document.getElementsByName('RECIPE_PADRAOQUALIDADE').length > 0) {
      const grid: GridEditable = Metadata.getInstance('RECIPE_PADRAOQUALIDADE');
      const form = Metadata.getInstance<Form>('FORM_PADRAOQUALIDADE');
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      const currentRow = plateTree.datasource.currentRow;
      if (form.validate()) {
        const padraoQual = form.value.NMPADRAOQUALIDADE.NRPADRAOQUALIDADE;
        const conditions = [
          {
            name: 'CDPRATO',
            operator: '=',
            value: currentRow.CDPRATO,
          },
          {
            name: 'CDFILAUXPRAT',
            operator: '=',
            value: currentRow.CDFILAUXPRAT,
          },
          {
            name: 'NRPADRAOQUALIDADE',
            operator: '=',
            value: padraoQual,
          },
        ];
        grid.datasource.filter = conditions;
        // validar
        const addButton: Button = grid.getChildInstance('addButton');
        if (!padraoQual) {
          addButton.disabled = true;
        } else {
          grid.datasource.get().then(() => {
            addButton.disabled = false;
          });
        }
      }
    }
  }
  
  // Modal Importação
  
  public async createImportModal() {
    // this.receitaController.setTabPlate();
    this.modalImport = await ModalService.createFromJson('/modals/importacaoReceita', true);
  }
    
  public showModalImport({ component }: IDictionary) {
    this.modalImport.show();
    const name = component.name;
    if (name.includes('PADRAO') > 0) this.recipeDest = 'PADRAO';
    else if (name.includes('TIPOCARD') > 0) this.recipeDest = 'TIPOCARD';
    else if (name.includes('LOCAL') > 0) this.recipeDest = 'LOCAL';
    else this.recipeDest = 'SERVICO';

    // const gridDest = this.utilities.returnGrid(component) as GridEditable;
    // this.prodDest = gridDest.datasource.data;
  }
  
  /* public closeModalImport() {
    this.modalImport.hide();
  } */
  
  // Dados da Receita
  
  /* public get visibleField() {
    const nameTab = this.setNameTab;
    if (nameTab === 'subTabReceitaRegiao' || nameTab === 'subTabReceitaPadraoQualidade') return false;
    return true;
  } */
  
  public setNameTabRecipe({ component }: IEventParam<Tabs>) {
    if (component.name === 'tabPratos' && component) {
      if (component.activeTab === 2) this.setNameTab = 'subTabReceitaPadrao';
      else if (component.activeTab === 3) this.setNameTab = 'subTabReceitaTipoCard';
      else if (component.activeTab === 4) this.setNameTab = 'subTabReceitaLocal';
      else if (component.activeTab === 5) this.setNameTab = 'subTabReceitaServico';
      else if (component.activeTab === 6) this.setNameTab = 'subTabReceitaPadraoQualidade';
      else if (component.activeTab === 7) this.setNameTab = 'subTabReceitaRegiao';
    } else this.setNameTab = component.name;
    
    this.dadosReceita.setNameTabRecipe();
    this.setCalculateNDPCAL();
  }
  
  public setCalculateNDPCAL() {
    const tab = this.setNameTab;
    if (tab === 'subTabReceitaPadrao') {
      const subTabReceitaPadrao = Metadata.getInstance<Tabs>('subTabReceitaPadrao');
      if (subTabReceitaPadrao.activeTab === 1) this.dadosReceita.setValuesRecipe();
      else if (subTabReceitaPadrao.activeTab === 2) this.consultaCustos.filterPlatesGrid();
      else if (subTabReceitaPadrao.activeTab === 3) this.consultaNutrientes.calcularNDPCAL();
      else if (subTabReceitaPadrao.activeTab === 4) this.modoPreparo.setFilters();
    } else if (tab === 'subTabReceitaTipoCard') {
      const subTabReceitaTipoCard = Metadata.getInstance<Tabs>('subTabReceitaTipoCard');
      if (subTabReceitaTipoCard.activeTab === 0) this.setfilterRecipeTpCard();
      if (subTabReceitaTipoCard.activeTab === 1) this.dadosReceita.setValuesRecipe();
      else if (subTabReceitaTipoCard.activeTab === 2) this.consultaCustos.filterPlatesGrid();
      else if (subTabReceitaTipoCard.activeTab === 3) this.consultaNutrientes.calcularNDPCAL();
      else if (subTabReceitaTipoCard.activeTab === 4) this.modoPreparo.setFilters();
    } else if (tab === 'subTabReceitaLocal') {
      const subTabReceitaLocal = Metadata.getInstance<Tabs>('subTabReceitaLocal');
      if (subTabReceitaLocal.activeTab === 0) this.setFilterRecipeLocal();
      else if (subTabReceitaLocal.activeTab === 1) this.dadosReceita.setValuesRecipe();
      else if (subTabReceitaLocal.activeTab === 2) this.consultaCustos.filterPlatesGrid();
      else if (subTabReceitaLocal.activeTab === 3) this.consultaNutrientes.calcularNDPCAL();
      else if (subTabReceitaLocal.activeTab === 4) this.modoPreparo.setFilters();
    } else if (tab === 'subTabReceitaServico') {
      const subTabReceitaServico = Metadata.getInstance<Tabs>('subTabReceitaServico');
      if (subTabReceitaServico.activeTab === 1) this.dadosReceita.setValuesRecipe();
      else if (subTabReceitaServico.activeTab === 2) this.consultaCustos.filterPlatesGrid();
      else if (subTabReceitaServico.activeTab === 3) this.consultaNutrientes.calcularNDPCAL();
      const subTabReceitaPadrao = Metadata.getInstance<Tabs>('subTabReceitaPadrao');
      if (subTabReceitaPadrao.activeTab === 1) this.dadosReceita.setValuesRecipe();
      else if (subTabReceitaPadrao.activeTab === 2) this.consultaCustos.filterPlatesGrid();
      else if (subTabReceitaPadrao.activeTab === 3) this.consultaNutrientes.calcularNDPCAL();
      else if (subTabReceitaPadrao.activeTab === 4) this.modoPreparo.setFilters();
    } else if (tab === 'subTabReceitaTipoCard') {
      const subTabReceitaTipoCard = Metadata.getInstance<Tabs>('subTabReceitaTipoCard');
      if (subTabReceitaTipoCard.activeTab === 0) this.setfilterRecipeTpCard();
      if (subTabReceitaTipoCard.activeTab === 1) this.dadosReceita.setValuesRecipe();
      else if (subTabReceitaTipoCard.activeTab === 2) this.consultaCustos.filterPlatesGrid();
      else if (subTabReceitaTipoCard.activeTab === 3) this.consultaNutrientes.calcularNDPCAL();
      else if (subTabReceitaTipoCard.activeTab === 4) this.modoPreparo.setFilters();
    } else if (tab === 'subTabReceitaLocal') {
      const subTabReceitaLocal = Metadata.getInstance<Tabs>('subTabReceitaLocal');
      if (subTabReceitaLocal.activeTab === 0) this.setFilterRecipeLocal();
      else if (subTabReceitaLocal.activeTab === 1) this.dadosReceita.setValuesRecipe();
      else if (subTabReceitaLocal.activeTab === 2) this.consultaCustos.filterPlatesGrid();
      else if (subTabReceitaLocal.activeTab === 3) this.consultaNutrientes.calcularNDPCAL();
      else if (subTabReceitaLocal.activeTab === 4) this.modoPreparo.setFilters();
    } else if (tab === 'subTabReceitaServico') {
      const subTabReceitaServico = Metadata.getInstance<Tabs>('subTabReceitaServico');
      if (subTabReceitaServico.activeTab === 1) this.dadosReceita.setValuesRecipe();
      else if (subTabReceitaServico.activeTab === 2) this.consultaCustos.filterPlatesGrid();
      else if (subTabReceitaServico.activeTab === 3) this.consultaNutrientes.calcularNDPCAL();
      else if (subTabReceitaServico.activeTab === 4) this.modoPreparo.setFilters();
    } else if (tab === 'subTabReceitaPadraoQualidade') {
      const subTabReceitaPadraoQualidade = Metadata.getInstance<Tabs>('subTabReceitaPadraoQualidade');
      if (subTabReceitaPadraoQualidade.activeTab === 1) this.dadosReceita.setValuesRecipe();
      else if (subTabReceitaPadraoQualidade.activeTab === 2) this.consultaCustos.filterPlatesGrid();
      else if (subTabReceitaPadraoQualidade.activeTab === 3) this.consultaNutrientes.calcularNDPCAL();
    } else if (tab === 'subTabReceitaRegiao') {
      const subTabReceitaRegiao = Metadata.getInstance<Tabs>('subTabReceitaRegiao');
      if (subTabReceitaRegiao.activeTab === 1) this.dadosReceita.setValuesRecipe();
      else if (subTabReceitaRegiao.activeTab === 2) this.consultaCustos.filterPlatesGrid();
      else if (subTabReceitaRegiao.activeTab === 3) this.consultaNutrientes.calcularNDPCAL();
    }
  }
  
  public validOperator() {
    const component: Tabs = Metadata.getInstance('tabPratos');
    const padrao: any = 'P';
    
    const row = {
      COLUMN_NAME: 'CDGROPERREPA',
      ALIAS_NAME: 'ALTRECEPADRAO',
    };
    
    if ((component.activeTab === 2 && this.cdGroupOperRepa === '') || (padrao === 'P' && this.cdGroupOperRepa === '')) {
      Http.post('/prato/returnParamsAddPlate', { requestType: 'Row', row }).then((callback) => {
        const result = callback.data.dataset.permission.result;
        if (result === 'NOPERMISSION') this.operatorValid = false;
        else this.operatorValid = true;
        this.cdGroupOperRepa = callback.data.dataset.paramgeral[0].CDGROPERREPA;
      });
    }
    
    if (component.activeTab === 1) {
      this.setfilterEmbalagem();
    }
    
    this.setNameTabRecipe({ component });
    this.createModalGroupOperador();
  }
  
  public searchOpLogado({ component }: Button) {
    const tab = this.setNameTab;
    
    let padrao: any;
    if (component === 'ZdSelect') padrao = 'P';
    
    if (tab === 'subTabReceitaPadrao' || padrao === 'P') {
      if (this.operatorValid === false) {
        const row = {
          CDOPERADOR: localStorage.getItem('CDOPERADOR'),
        };
        Http.post('/prato/operGrup', { requestType: 'Row', row }).then((callback) => {
          const resultCodGrupo = callback.data.dataset.result[0]?.CDGRUPOPER;
          if (resultCodGrupo !== this.cdGroupOperRepa) {
            this.utilities.showAlert('Alert', 'pme::modificacaoRecPadrao', 'warning');
            this.openModalGroupOperador();
          }
        });
      } else this.receita.addRow({ component });
    } else if (tab === 'subTabReceitaPadrao' && this.operatorValid === true) this.receita.addRow({ component });
  }
  
  private async createModalGroupOperador() {
    this.modalGroupOperador = await ModalService.createFromJson('planejamento/prato/modals/formGroupOperador', true);
  }

  public closeModalGroupOperador() {
    this.modalGroupOperador.hide();
  }
  
  public openModalGroupOperador() {
    if (!this.modalGroupOperador) {
      this.createModalGroupOperador().then(() => {
        this.modalGroupOperador.show();
      });
    } else {
      this.modalGroupOperador.show();
    }
  }
  
  // validar
  public saveOperator({ component }: Button) {
    const tab = this.setNameTab;
    const form = Metadata.getInstance<Form>('formGroupOper');
    const row = form.value;
    row.EMAIL = row.DSEMAILOPER;
    row.SENHA = row.CDSENHAOPERWEB;
     
    if (form.validate()) {
      this.utilities.activeLoadingButton(component);
      Http.post('/prato/autenticar', { requestType: 'Row', row }).then((permission) => {
        if (permission.data.dataset.permission.isValid !== false) {
          Http.post('/prato/saveOperator', { requestType: 'Row', row }).then((result) => {
            const msg = result.data.dataset.result.message;
            if (msg === 'INVALID_PASSWORD' || msg === 'operadorNaoCadastrado') {
              this.utilities.showAlert('Alert', 'pme::ERROR_SENHAUSUARIONEGADO', 'error');
              this.operatorValid = false;
            } else if (msg === null) {
              this.operatorValid = true;
              this.closeModalGroupOperador();
              if (tab === 'subTabReceitaPadrao') {
                const subTabReceitaPadrao = Metadata.getInstance<Tabs>('subTabReceitaPadrao');
                if (subTabReceitaPadrao.activeTab === 1) {
                  const button: Button = Metadata.getInstance('saveButtonRecipePadrao');
                  this.dadosReceita.salvaDadosRecipePlate(button);
                }
              }
            } else this.operatorValid = false;
          }).catch((erro: string) => {
            this.utilities.showAlert('alert', `pme::${erro}`, 'error');
          });
        } else this.utilities.showAlert('alert', 'pme::unauthorizedOperator', 'error');
      }).catch((erro: string) => {
        this.utilities.showAlert('alert', `pme::${erro}`, 'error');
      });
    }
  }
  
  /* public setFilterAgregAgrup({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    
    const conditions = [
      {
        name: 'P_CDGRUPFILI',
        operator: '=',
        value: agrupUnid.value?.CDGRUPFILI ? agrupUnid.value.CDGRUPFILI : 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  } */
  
  /* public setFilterServAgrupUnid({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const unidadeValue = (unidade.value.length !== 0) ? unidade.value.join('_') : 'T';
    
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const agrupUnidValue = agrupUnid.value ? agrupUnid.value.CDGRUPFILI : 'T';
    
    const grupserv = Metadata.getInstance<Select>('GRUPOSERVICO');
    const grupServValue = grupserv.value ? grupserv.value : 'T';
    
    const filter = [
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidadeValue,
      },
      {
        name: 'CDGRUPSERV',
        operator: '=',
        value: grupServValue,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: this.project === 'PAE' ? '' : 'N',
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  } */
  
  /* public controllPratoIni() {
    const pratoFinal = Metadata.getInstance<Select>('PRATO_FINAL');
    if (pratoFinal.value !== null) pratoFinal.setValue(null);
  } */
  
  /* public verifyPratIni({ component }: IEventParam<Select>) {
    const form = Metadata.getInstance<Form>('formFilter');
    if (component.value !== null) {
      if (this.utilities.returnPrat(form.getChildInstance<Select>('PRATO_INICIAL').value, component.value)) {
        this.utilities.showAlert('Alert', 'pme::pratoIniMaior', 'error');
        component.setValue(null);
      }
    }
  } */
  
  public getSafColor({ row }: IDictionary) {
    let decColor = 0;
    const nrcorsafra = this.utilities.hexToRgbInline(this.nrcorsafra).split('.', 3);
    const nrcorinicsaf = this.utilities.hexToRgbInline(this.nrcorinicsaf).split('.', 3);
    const nrcorentrsaf = this.utilities.hexToRgbInline(this.nrcorentrsaf).split('.', 3);
    
    if (row.IDINTESAFR > 0) {
      if (row.IDINTESAFR === '1') {
        decColor = this.utilities.RGBToTColor(nrcorsafra[0], nrcorsafra[1], nrcorsafra[2]);
        row.SAFRA = I18n.translate('pme::safra');
      }
      if (row.IDINTESAFR === '2') {
        decColor = this.utilities.RGBToTColor(nrcorinicsaf[0], nrcorinicsaf[1], nrcorinicsaf[2]);
        row.SAFRA = I18n.translate('pme::inicioFimSafra');
      }
      if (row.IDINTESAFR === '3') {
        decColor = this.utilities.RGBToTColor(nrcorentrsaf[0], nrcorentrsaf[1], nrcorentrsaf[2]);
        row.SAFRA = I18n.translate('pme::entreSafra');
      }
    } else row.IDINTESAFR = '';
    
    const color = this.utilities.TColorToHex(decColor.toString());
    
    return color;
  }
  
  // verificação op logado modo preparo
  public addRowModoPrep() {
    const tabPratos: Tabs = Metadata.getInstance('tabPratos');
    if (tabPratos.activeTab === 2) {
      if (this.operatorValid !== false) {
        this.modoPreparo.addRow();
      } else {
        this.utilities.showAlert('Alert', 'pme::modificacaoRecPadrao', 'warning');
        this.openModalGroupOperador();
      }
    } else this.modoPreparo.addRow();
  }
  
  // verificacao op logados dados receita padrao
  
  public salvaDadosRecipePlate() {
    if (this.operatorValid !== false) {
      const component: Button = Metadata.getInstance('saveButtonRecipePadrao');
      this.dadosReceita.salvaDadosRecipePlate(component);
    } else {
      this.utilities.showAlert('Alert', 'pme::modificacaoRecPadrao', 'warning');
      this.openModalGroupOperador();
    }
  }
  
  // verificação op logado receita padrao
  
  public deleteRecipeStandardPlate() {
    const buttonDelete = Metadata.getInstance<Button>('delButtonPad');
    if (this.operatorValid !== false) {
      this.receita.deleteRecipeStandard({ component: buttonDelete });
    } else {
      this.utilities.showAlert('Alert', 'pme::modificacaoRecPadrao', 'warning');
      this.openModalGroupOperador();
    }
  }
  
  public onInlineEditRecipeStandart() {
    const grid: Grid = Metadata.getInstance('RECIPE_STANDARD');
    const toolbarSlot = grid.toolbarSlot;
    
    if (this.operatorValid !== false) {
      this.gridEditController.onInlineEdit({ component: grid });
    } else {
      this.utilities.showAlert('Alert', 'pme::modificacaoRecPadrao', 'warning');
      this.openModalGroupOperador();
      this.gridEditController.reload({ component: grid });
      toolbarSlot.forEach((e: IComponentRender) => {
        if (e.name === 'cancelButton') e.disabled = !e.disabled;
        if (e.name === 'saveButton') e.disabled = !e.disabled;
      });
    }
  }
  
  // modal emissao receitas
  
  private async createModalEmissaoReceitas() {
    this.modalEmissaoReceitas = await ModalService.createFromJson('planejamento/prato/modals/pla01303_emissao_receitas', true);
  }

  public closeModalEmissaoReceitas() {
    this.modalEmissaoReceitas.hide();
  }
  
  public openModalEmissaoReceitas() {
    if (!this.modalEmissaoReceitas) {
      this.createModalEmissaoReceitas().then(() => {
        this.modalEmissaoReceitas.show();
      });
    } else {
      this.modalEmissaoReceitas.show();
    }
  }
}
