import {
  ModalService, Modal, Form, Select, Grid, IButtonEvent, GridEditable, IComponentRender,
} from '@zeedhi/common';
import {
  Loader, Metadata, IEventParam, IDictionary,
} from '@zeedhi/core';

export class Pla01313ReplicacaoReceitasServicoController {
  private utilities = Loader.getInstance('Utilities');
  
  public modalReplicacaoReceitasServico!: Modal;
  
  private async createModalReplicacaoReceitasServico() {
    this.modalReplicacaoReceitasServico = await ModalService.createFromJson('planejamento/prato/modals/pla01313_replicacao_receitas_servico', true);
  }

  public closeModalReplicacaoReceitasServico() {
    this.modalReplicacaoReceitasServico.hide();
  }
  
  public openModalReplicacaoReceitasServico() {
    if (!this.modalReplicacaoReceitasServico) {
      this.createModalReplicacaoReceitasServico().then(() => {
        this.modalReplicacaoReceitasServico.show();
      });
    } else {
      this.modalReplicacaoReceitasServico.show();
    }
  }
  
  public fullscreen({ component }: IButtonEvent) {
    this.modalReplicacaoReceitasServico.fullscreen = !this.modalReplicacaoReceitasServico.fullscreen;
    component.iconName = (this.modalReplicacaoReceitasServico.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }
  
  public inicContainerOrigem() {
    const project = localStorage.getItem('PRODUCT_ID');
    
    const tpReceitaCard = Metadata.getInstance<Select>('TPRECEITA');
    const tpReceita = Metadata.getInstance<Select>('TPRECEITA1');
    const cardapio = Metadata.getInstance<Select>('TIPOCARD');

    if (project === '4875') {
      tpReceitaCard.isVisible = true;
      tpReceita.isVisible = false;
    } else {
      tpReceitaCard.isVisible = false;
      tpReceita.isVisible = true;
      cardapio.isVisible = false;
    }
  }
  
  public setFilterService({ component }: IEventParam<Select>) {
    const form: Form = Metadata.getInstance('formAcoesPrato');
    const row = form.value;
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: row.UNIDADE,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: localStorage.getItem('PRODUCT_ID') === '4875' ? '' : 'N',
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public setFilterServicos() {
    const form: Form = Metadata.getInstance('formAcoesPrato2');
    const row = form.value;
    const grid: Grid = Metadata.getInstance('gridServico');
  
    if (form.validate()) {
      const conditions = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: row.UNIDADEDEST !== undefined ? row.UNIDADEDEST : 'T',
        },
        {
          name: 'CDGRUPSERV',
          operator: '=',
          value: row.GRUPOSERVICO !== null ? row.GRUPOSERVICO : 'T',
        },
      ];
      grid.datasource.filter = conditions;
      grid.datasource.get();
    } else grid.datasource.data = [];
  }
  
  public saveReplicacaoServicos() {
    const form = Metadata.getInstance<Form>('formAcoesPrato2');
    const grid: GridEditable = Metadata.getInstance('gridServico');
    const editedRows = grid.getEditedRows() as IDictionary;

    if (form.validate() && editedRows.length > 0) {
      this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      this.atualizarGridReplicacaoServicos();
    }
  }
  
  public atualizarGridReplicacaoServicos() {
    const grid: GridEditable = Metadata.getInstance('gridServico');
    const editedRows = grid.getEditedRows() as IDictionary;
    const newPorc = editedRows[0].PERCENTUAL;
    grid.datasource.currentRow.PERCENTUAL = newPorc;
    grid.editing = false;
    
    const toolbarSlot = grid.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton') e.disabled = !e.disabled;
      if (e.name === 'saveButton') e.disabled = !e.disabled;
    });
  }
  
  public confirmarAcao() {
    const form: Form = Metadata.getInstance('formAcoesPrato');
    const row = form.value;
    const gridPrato: Grid = Metadata.getInstance('gridPrato');
    const rowsGridPrato = gridPrato.selectedRows;
    const gridServico: Grid = Metadata.getInstance('gridServico');
    const rowsGridServico = gridServico.selectedRows;
    
    if (row.TPRECEITA === null) row.TPRECEITA = row.TPRECEITA1;
    
    if (form.validate()) {
      if (rowsGridPrato.length === 0) this.utilities.showAlert('Alert', 'pme::aoMenosUmPratoSelecionado', 'error');
      else if (rowsGridServico.length === 0) this.utilities.showAlert('Alert', 'pme::aoMenosUmServicoSelecionado', 'error');
      else {
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        this.closeModalReplicacaoReceitasServico();
      }
    }
  }
}
